@@ -68,14 +68,14 @@ typedef unsigned short ushort;
 #endif
 
 /* helper tables */
-extern const uchar icvSaturate8u[];
-#define CV_FAST_CAST_8U(t)  (assert(-256 <= (t) || (t) <= 512), icvSaturate8u[(t)+256])
+extern const uchar icvSaturate8u_cv[];
+#define CV_FAST_CAST_8U(t)  (assert(-256 <= (t) || (t) <= 512), icvSaturate8u_cv[(t)+256])
 #define CV_CALC_MIN_8U(a,b) (a) -= CV_FAST_CAST_8U((a) - (b))
 #define CV_CALC_MAX_8U(a,b) (a) += CV_FAST_CAST_8U((b) - (a))
 
-// -128.f ... 255.f
+// -256.f ... 511.f
 extern const float icv8x32fTab_cv[];
-#define CV_8TO32F(x)  icv8x32fTab_cv[(x)+128]
+#define CV_8TO32F(x)  icv8x32fTab_cv[(x)+256]
 
 // (-128.f)^2 ... (255.f)^2
 extern const float icv8x32fSqrTab[];
