@@ -47,61 +47,6 @@
 
 #include "_cxcore.h"
 
-static const uchar icvSaturate8u[] = 
-{
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
-      0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11,  12,  13,  14,  15,
-     16,  17,  18,  19,  20,  21,  22,  23,  24,  25,  26,  27,  28,  29,  30,  31,
-     32,  33,  34,  35,  36,  37,  38,  39,  40,  41,  42,  43,  44,  45,  46,  47,
-     48,  49,  50,  51,  52,  53,  54,  55,  56,  57,  58,  59,  60,  61,  62,  63,
-     64,  65,  66,  67,  68,  69,  70,  71,  72,  73,  74,  75,  76,  77,  78,  79,
-     80,  81,  82,  83,  84,  85,  86,  87,  88,  89,  90,  91,  92,  93,  94,  95,
-     96,  97,  98,  99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
-    112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127,
-    128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140, 141, 142, 143,
-    144, 145, 146, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 159,
-    160, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174, 175,
-    176, 177, 178, 179, 180, 181, 182, 183, 184, 185, 186, 187, 188, 189, 190, 191,
-    192, 193, 194, 195, 196, 197, 198, 199, 200, 201, 202, 203, 204, 205, 206, 207,
-    208, 209, 210, 211, 212, 213, 214, 215, 216, 217, 218, 219, 220, 221, 222, 223,
-    224, 225, 226, 227, 228, 229, 230, 231, 232, 233, 234, 235, 236, 237, 238, 239,
-    240, 241, 242, 243, 244, 245, 246, 247, 248, 249, 250, 251, 252, 253, 254, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
-    255
-};
-
-#define CV_FAST_CAST_8U(t)   (assert(-256 <= (t) && (t) <= 512), icvSaturate8u[(t)+256])
-
 /****************************************************************************************\
 *                      Arithmetic operations (+, -) without mask                         *
 \****************************************************************************************/
@@ -138,11 +83,11 @@ IPCVAPI_IMPL( CvStatus, name,                                               \
       type* dst, int step, CvSize size ),                                   \
       (src1, step1, src2, step2, dst, step, size) )                         \
 {                                                                           \
+    step1/=sizeof(src1[0]); step2/=sizeof(src2[0]); step/=sizeof(dst[0]);   \
+                                                                            \
     if( size.width == 1 )                                                   \
     {                                                                       \
-        for( ; size.height--; (char*&)src1 += step1,                        \
-                              (char*&)src2 += step2,                        \
-                              (char*&)dst += step )                         \
+        for( ; size.height--; src1 += step1, src2 += step2, dst += step )   \
         {                                                                   \
             worktype t0 = __op__((src1)[0],(src2)[0]);                      \
             (dst)[0] = cast_macro( t0 );                                    \
@@ -150,9 +95,7 @@ IPCVAPI_IMPL( CvStatus, name,                                               \
     }                                                                       \
     else                                                                    \
     {                                                                       \
-        for( ; size.height--; (char*&)src1 += step1,                        \
-                              (char*&)src2 += step2,                        \
-                              (char*&)dst += step )                         \
+        for( ; size.height--; src1 += step1, src2 += step2, dst += step )   \
         {                                                                   \
             ICV_DEF_BIN_ARI_OP_CASE( __op__, worktype,                      \
                                      cast_macro, size.width );              \
@@ -169,11 +112,11 @@ IPCVAPI_IMPL( CvStatus, name,                                               \
       type* dst, int step, CvSize size, int /*scalefactor*/ ),              \
       (src1, step1, src2, step2, dst, step, size, 0) )                      \
 {                                                                           \
+    step1/=sizeof(src1[0]); step2/=sizeof(src2[0]); step/=sizeof(dst[0]);   \
+                                                                            \
     if( size.width == 1 )                                                   \
     {                                                                       \
-        for( ; size.height--; (char*&)src1 += step1,                        \
-                              (char*&)src2 += step2,                        \
-                              (char*&)dst += step )                         \
+        for( ; size.height--; src1 += step1, src2 += step2, dst += step )   \
         {                                                                   \
             worktype t0 = __op__((src1)[0],(src2)[0]);                      \
             (dst)[0] = cast_macro( t0 );                                    \
@@ -181,9 +124,7 @@ IPCVAPI_IMPL( CvStatus, name,                                               \
     }                                                                       \
     else                                                                    \
     {                                                                       \
-        for( ; size.height--; (char*&)src1 += step1,                        \
-                              (char*&)src2 += step2,                        \
-                              (char*&)dst += step )                         \
+        for( ; size.height--; src1 += step1, src2 += step2, dst += step )   \
         {                                                                   \
             ICV_DEF_BIN_ARI_OP_CASE( __op__, worktype,                      \
                                      cast_macro, size.width );              \
@@ -251,10 +192,11 @@ static CvStatus CV_STDCALL name                                             \
     ( const type* src, int step1, type* dst, int step,                      \
       CvSize size, const worktype* scalar )                                 \
 {                                                                           \
+    step1 /= sizeof(src[0]); step /= sizeof(dst[0]);                        \
+                                                                            \
     if( size.width == 1 )                                                   \
     {                                                                       \
-        for( ; size.height--; (char*&)src += step1,                         \
-                              (char*&)dst += step )                         \
+        for( ; size.height--; src += step1, dst += step )                   \
         {                                                                   \
             worktype t0 = __op__(*(scalar),*(src));                         \
             *(dst) = cast_macro( t0 );                                      \
@@ -262,8 +204,7 @@ static CvStatus CV_STDCALL name                                             \
     }                                                                       \
     else                                                                    \
     {                                                                       \
-        for( ; size.height--; (char*&)src += step1,                         \
-                              (char*&)dst += step )                         \
+        for( ; size.height--; src += step1, dst += step )                   \
         {                                                                   \
             const type *tsrc = src;                                         \
             type *tdst = dst;                                               \
@@ -448,8 +389,6 @@ cvSub( const void* srcarr1, const void* srcarr2,
                     const float* src2data = (const float*)(src2->data.ptr);
                     float* dstdata = (float*)(dst->data.ptr);
 
-                    size.width *= size.height*cn;
-
                     do
                     {
                         dstdata[len-1] = (float)(src2data[len-1] - src1data[len-1]);
@@ -577,7 +516,7 @@ cvSub( const void* srcarr1, const void* srcarr2,
     __END__;
 
     if( !local_alloc )
-        cvFree( (void**)&buffer );
+        cvFree( &buffer );
 }
 
 
@@ -798,7 +737,7 @@ cvSubRS( const void* srcarr, CvScalar scalar, void* dstarr, const void* maskarr
     __END__;
 
     if( !local_alloc )
-        cvFree( (void**)&buffer );
+        cvFree( &buffer );
 }
 
 
@@ -922,8 +861,6 @@ cvAdd( const void* srcarr1, const void* srcarr2,
                     const float* src2data = (const float*)(src2->data.ptr);
                     float* dstdata = (float*)(dst->data.ptr);
 
-                    size.width *= size.height*cn;
-
                     do
                     {
                         dstdata[len-1] = (float)(src1data[len-1] + src2data[len-1]);
@@ -1051,7 +988,7 @@ cvAdd( const void* srcarr1, const void* srcarr2,
     __END__;
 
     if( !local_alloc )
-        cvFree( (void**)&buffer );
+        cvFree( &buffer );
 }
 
 
@@ -1272,7 +1209,7 @@ cvAddS( const void* srcarr, CvScalar scalar, void* dstarr, const void* maskarr )
     __END__;
 
     if( !local_alloc )
-        cvFree( (void**)&buffer );
+        cvFree( &buffer );
 }
 
 
@@ -1286,11 +1223,11 @@ static CvStatus CV_STDCALL
                            arrtype* dst, int step,                                      \
                            CvSize size, double scale )                                  \
 {                                                                                       \
+    step1 /= sizeof(src1[0]); step2 /= sizeof(src2[0]); step /= sizeof(dst[0]);         \
+                                                                                        \
     if( fabs(scale - 1.) < DBL_EPSILON )                                                \
     {                                                                                   \
-        for( ; size.height--; (char*&)src1+=step1,                                      \
-                              (char*&)src2+=step2,                                      \
-                              (char*&)dst+=step )                                       \
+        for( ; size.height--; src1+=step1, src2+=step2, dst+=step )                     \
         {                                                                               \
             int i;                                                                      \
             for( i = 0; i <= size.width - 4; i += 4 )                                   \
@@ -1317,9 +1254,7 @@ static CvStatus CV_STDCALL
     }                                                                                   \
     else                                                                                \
     {                                                                                   \
-        for( ; size.height--; (char*&)src1+=step1,                                      \
-                              (char*&)src2+=step2,                                      \
-                              (char*&)dst+=step )                                       \
+        for( ; size.height--; src1+=step1, src2+=step2, dst+=step )                     \
         {                                                                               \
             int i;                                                                      \
             for( i = 0; i <= size.width - 4; i += 4 )                                   \
@@ -1532,8 +1467,8 @@ cvMul( const void* srcarr1, const void* srcarr2, void* dstarr, double scale )
 
 /***************************************** D I V ****************************************/
 
-#define ICV_DEF_DIV_OP_CASE( flavor, arrtype, worktype, _cast_macro1_,                  \
-                             _cast_macro2_, _cvt_macro_, _check_macro_ )                \
+#define ICV_DEF_DIV_OP_CASE( flavor, arrtype, worktype, checktype, _start_row_macro_,   \
+    _cast_macro1_, _cast_macro2_, _cvt_macro_, _check_macro_, isrc )                    \
                                                                                         \
 static CvStatus CV_STDCALL                                                              \
 icvDiv_##flavor##_C1R( const arrtype* src1, int step1,                                  \
@@ -1541,13 +1476,15 @@ icvDiv_##flavor##_C1R( const arrtype* src1, int step1,
                        arrtype* dst, int step,                                          \
                        CvSize size, double scale )                                      \
 {                                                                                       \
-    for( ; size.height--; (char*&)src1+=step1, (char*&)src2+=step2, (char*&)dst+=step ) \
+    step1 /= sizeof(src1[0]); step2 /= sizeof(src2[0]); step /= sizeof(dst[0]);         \
+                                                                                        \
+    for( ; size.height--; src1+=step1, src2+=step2, dst+=step )                         \
     {                                                                                   \
-        int i;                                                                          \
+        _start_row_macro_(checktype, src2);                                             \
         for( i = 0; i <= size.width - 4; i += 4 )                                       \
         {                                                                               \
-            if( _check_macro_(src2[i]) && _check_macro_(src2[i+1]) &&                   \
-                _check_macro_(src2[i+2]) && _check_macro_(src2[i+3]))                   \
+            if( _check_macro_(isrc[i]) && _check_macro_(isrc[i+1]) &&                   \
+                _check_macro_(isrc[i+2]) && _check_macro_(isrc[i+3]))                   \
             {                                                                           \
                 double a = (double)_cvt_macro_(src2[i]) * _cvt_macro_(src2[i+1]);       \
                 double b = (double)_cvt_macro_(src2[i+2]) * _cvt_macro_(src2[i+3]);     \
@@ -1568,13 +1505,13 @@ icvDiv_##flavor##_C1R( const arrtype* src1, int step1,
             }                                                                           \
             else                                                                        \
             {                                                                           \
-                worktype z0 = _check_macro_(src2[i]) ?                                  \
+                worktype z0 = _check_macro_(isrc[i]) ?                                  \
                    _cast_macro1_(_cvt_macro_(src1[i])*scale/_cvt_macro_(src2[i])) : 0;  \
-                worktype z1 = _check_macro_(src2[i+1]) ?                                \
+                worktype z1 = _check_macro_(isrc[i+1]) ?                                \
                    _cast_macro1_(_cvt_macro_(src1[i+1])*scale/_cvt_macro_(src2[i+1])):0;\
-                worktype z2 = _check_macro_(src2[i+2]) ?                                \
+                worktype z2 = _check_macro_(isrc[i+2]) ?                                \
                    _cast_macro1_(_cvt_macro_(src1[i+2])*scale/_cvt_macro_(src2[i+2])):0;\
-                worktype z3 = _check_macro_(src2[i+3]) ?                                \
+                worktype z3 = _check_macro_(isrc[i+3]) ?                                \
                    _cast_macro1_(_cvt_macro_(src1[i+3])*scale/_cvt_macro_(src2[i+3])):0;\
                                                                                         \
                 dst[i] = _cast_macro2_(z0);                                             \
@@ -1586,9 +1523,8 @@ icvDiv_##flavor##_C1R( const arrtype* src1, int step1,
                                                                                         \
         for( ; i < size.width; i++ )                                                    \
         {                                                                               \
-            worktype z0 = _check_macro_(src2[i]) ?                                      \
+            worktype z0 = _check_macro_(isrc[i]) ?                                      \
                 _cast_macro1_(_cvt_macro_(src1[i])*scale/_cvt_macro_(src2[i])) : 0;     \
-                                                                                        \
             dst[i] = _cast_macro2_(z0);                                                 \
         }                                                                               \
     }                                                                                   \
@@ -1597,21 +1533,24 @@ icvDiv_##flavor##_C1R( const arrtype* src1, int step1,
 }
 
 
-#define ICV_DEF_RECIP_OP_CASE( flavor, arrtype, worktype, _cast_macro1_,        \
-                             _cast_macro2_, _cvt_macro_, _check_macro_ )        \
+#define ICV_DEF_RECIP_OP_CASE( flavor, arrtype, worktype, checktype,            \
+    _start_row_macro_, _cast_macro1_, _cast_macro2_,                            \
+    _cvt_macro_, _check_macro_, isrc )                                          \
                                                                                 \
 static CvStatus CV_STDCALL                                                      \
 icvRecip_##flavor##_C1R( const arrtype* src, int step1,                         \
                          arrtype* dst, int step,                                \
                          CvSize size, double scale )                            \
 {                                                                               \
-    for( ; size.height--; (char*&)src+=step1, (char*&)dst+=step )               \
+    step1 /= sizeof(src[0]); step /= sizeof(dst[0]);                            \
+                                                                                \
+    for( ; size.height--; src+=step1, dst+=step )                               \
     {                                                                           \
-        int i;                                                                  \
+        _start_row_macro_(checktype, src);                                      \
         for( i = 0; i <= size.width - 4; i += 4 )                               \
         {                                                                       \
-            if( _check_macro_(src[i]) && _check_macro_(src[i+1]) &&             \
-                _check_macro_(src[i+2]) && _check_macro_(src[i+3]))             \
+            if( _check_macro_(isrc[i]) && _check_macro_(isrc[i+1]) &&           \
+                _check_macro_(isrc[i+2]) && _check_macro_(isrc[i+3]))           \
             {                                                                   \
                 double a = (double)_cvt_macro_(src[i]) * _cvt_macro_(src[i+1]); \
                 double b = (double)_cvt_macro_(src[i+2]) * _cvt_macro_(src[i+3]);\
@@ -1632,13 +1571,13 @@ icvRecip_##flavor##_C1R( const arrtype* src, int step1,
             }                                                                   \
             else                                                                \
             {                                                                   \
-                worktype z0 = _check_macro_(src[i]) ?                           \
+                worktype z0 = _check_macro_(isrc[i]) ?                          \
                    _cast_macro1_(scale/_cvt_macro_(src[i])) : 0;                \
-                worktype z1 = _check_macro_(src[i+1]) ?                         \
+                worktype z1 = _check_macro_(isrc[i+1]) ?                        \
                    _cast_macro1_(scale/_cvt_macro_(src[i+1])):0;                \
-                worktype z2 = _check_macro_(src[i+2]) ?                         \
+                worktype z2 = _check_macro_(isrc[i+2]) ?                        \
                    _cast_macro1_(scale/_cvt_macro_(src[i+2])):0;                \
-                worktype z3 = _check_macro_(src[i+3]) ?                         \
+                worktype z3 = _check_macro_(isrc[i+3]) ?                        \
                    _cast_macro1_(scale/_cvt_macro_(src[i+3])):0;                \
                                                                                 \
                 dst[i] = _cast_macro2_(z0);                                     \
@@ -1650,9 +1589,8 @@ icvRecip_##flavor##_C1R( const arrtype* src, int step1,
                                                                                 \
         for( ; i < size.width; i++ )                                            \
         {                                                                       \
-            worktype z0 = _check_macro_(src[i]) ?                               \
+            worktype z0 = _check_macro_(isrc[i]) ?                              \
                 _cast_macro1_(scale/_cvt_macro_(src[i])) : 0;                   \
-                                                                                \
             dst[i] = _cast_macro2_(z0);                                         \
         }                                                                       \
     }                                                                           \
@@ -1660,32 +1598,51 @@ icvRecip_##flavor##_C1R( const arrtype* src, int step1,
     return CV_OK;                                                               \
 }
 
-#define div_check_zero_flt(x)  (((int&)(x) & 0x7fffffff) != 0)
-#define div_check_zero_dbl(x)  (((int64&)(x) & CV_BIG_INT(0x7fffffffffffffff)) != 0)
+
+#define div_start_row_int(checktype, divisor) \
+    int i
+
+#define div_start_row_flt(checktype, divisor) \
+    const checktype* isrc = (const checktype*)divisor; int i
+
+#define div_check_zero_flt(x)  (((x) & 0x7fffffff) != 0)
+#define div_check_zero_dbl(x)  (((x) & CV_BIG_INT(0x7fffffffffffffff)) != 0)
 
 #if defined WIN64 && defined EM64T && defined _MSC_VER && !defined CV_ICC
 #pragma optimize("",off)
 #endif
 
-ICV_DEF_DIV_OP_CASE( 8u, uchar, int, cvRound, CV_CAST_8U, CV_8TO32F, CV_NONZERO )
+ICV_DEF_DIV_OP_CASE( 8u, uchar, int, uchar, div_start_row_int,
+                     cvRound, CV_CAST_8U, CV_8TO32F, CV_NONZERO, src2 )
 
 #if defined WIN64 && defined EM64T && defined _MSC_VER && !defined CV_ICC
 #pragma optimize("",on)
 #endif
 
 
-ICV_DEF_DIV_OP_CASE( 16u, ushort, int, cvRound, CV_CAST_16U, CV_CAST_64F, CV_NONZERO )
-ICV_DEF_DIV_OP_CASE( 16s, short, int, cvRound, CV_CAST_16S, CV_NOP, CV_NONZERO )
-ICV_DEF_DIV_OP_CASE( 32s, int, int, cvRound, CV_CAST_32S, CV_CAST_64F, CV_NONZERO )
-ICV_DEF_DIV_OP_CASE( 32f, float, double, CV_NOP, CV_CAST_32F, CV_NOP, div_check_zero_flt )
-ICV_DEF_DIV_OP_CASE( 64f, double, double, CV_NOP, CV_CAST_64F, CV_NOP, div_check_zero_dbl )
+ICV_DEF_DIV_OP_CASE( 16u, ushort, int, ushort, div_start_row_int,
+                     cvRound, CV_CAST_16U, CV_CAST_64F, CV_NONZERO, src2 )
+ICV_DEF_DIV_OP_CASE( 16s, short, int, short, div_start_row_int,
+                     cvRound, CV_CAST_16S, CV_NOP, CV_NONZERO, src2 )
+ICV_DEF_DIV_OP_CASE( 32s, int, int, int, div_start_row_int,
+                     cvRound, CV_CAST_32S, CV_CAST_64F, CV_NONZERO, src2 )
+ICV_DEF_DIV_OP_CASE( 32f, float, double, int, div_start_row_flt,
+                     CV_NOP, CV_CAST_32F, CV_NOP, div_check_zero_flt, isrc )
+ICV_DEF_DIV_OP_CASE( 64f, double, double, int64, div_start_row_flt,
+                     CV_NOP, CV_CAST_64F, CV_NOP, div_check_zero_dbl, isrc )
 
-ICV_DEF_RECIP_OP_CASE( 8u, uchar, int, cvRound, CV_CAST_8U, CV_8TO32F, CV_NONZERO )
-ICV_DEF_RECIP_OP_CASE( 16u, ushort, int, cvRound, CV_CAST_16U, CV_CAST_64F, CV_NONZERO )
-ICV_DEF_RECIP_OP_CASE( 16s, short, int, cvRound, CV_CAST_16S, CV_NOP, CV_NONZERO )
-ICV_DEF_RECIP_OP_CASE( 32s, int, int, cvRound, CV_CAST_32S, CV_CAST_64F, CV_NONZERO )
-ICV_DEF_RECIP_OP_CASE( 32f, float, double, CV_NOP, CV_CAST_32F, CV_NOP, div_check_zero_flt )
-ICV_DEF_RECIP_OP_CASE( 64f, double, double, CV_NOP, CV_CAST_64F, CV_NOP, div_check_zero_dbl )
+ICV_DEF_RECIP_OP_CASE( 8u, uchar, int, uchar, div_start_row_int,
+                       cvRound, CV_CAST_8U, CV_8TO32F, CV_NONZERO, src )
+ICV_DEF_RECIP_OP_CASE( 16u, ushort, int, ushort, div_start_row_int,
+                       cvRound, CV_CAST_16U, CV_CAST_64F, CV_NONZERO, src )
+ICV_DEF_RECIP_OP_CASE( 16s, short, int, short, div_start_row_int,
+                       cvRound, CV_CAST_16S, CV_NOP, CV_NONZERO, src )
+ICV_DEF_RECIP_OP_CASE( 32s, int, int, int, div_start_row_int,
+                       cvRound, CV_CAST_32S, CV_CAST_64F, CV_NONZERO, src )
+ICV_DEF_RECIP_OP_CASE( 32f, float, double, int, div_start_row_flt,
+                       CV_NOP, CV_CAST_32F, CV_NOP, div_check_zero_flt, isrc  )
+ICV_DEF_RECIP_OP_CASE( 64f, double, double, int64, div_start_row_flt,
+                       CV_NOP, CV_CAST_64F, CV_NOP, div_check_zero_dbl, isrc )
 
 ICV_DEF_INIT_ARITHM_FUNC_TAB( Div, C1R )
 ICV_DEF_INIT_ARITHM_FUNC_TAB( Recip, C1R )
@@ -1861,46 +1818,47 @@ cvDiv( const void* srcarr1, const void* srcarr2, void* dstarr, double scale )
 
 /******************************* A D D   W E I G T E D ******************************/
 
-#define ICV_DEF_ADD_WEIGHTED_OP(flavor, arrtype, worktype, load_macro,                  \
-                                     cast_macro1, cast_macro2)                          \
-static CvStatus CV_STDCALL                                                              \
-icvAddWeighted_##flavor##_C1R( const arrtype* src1, int step1, double alpha,            \
-                               const arrtype* src2, int step2, double beta,             \
-                               double gamma, arrtype* dst, int step, CvSize size )      \
-{                                                                                       \
-    for( ; size.height--; (char*&)src1 += step1, (char*&)src2 += step2,                 \
-                          (char*&)dst += step )                                         \
-    {                                                                                   \
-        int i;                                                                          \
-                                                                                        \
-        for( i = 0; i <= size.width - 4; i += 4 )                                       \
-        {                                                                               \
-            worktype t0 = cast_macro1(load_macro((src1)[i])*alpha +                     \
-                                      load_macro((src2)[i])*beta + gamma);              \
-            worktype t1 = cast_macro1(load_macro((src1)[i+1])*alpha +                   \
-                                      load_macro((src2)[i+1])*beta + gamma);            \
-                                                                                        \
-            (dst)[i] = cast_macro2( t0 );                                               \
-            (dst)[i+1] = cast_macro2( t1 );                                             \
-                                                                                        \
-            t0 = cast_macro1(load_macro((src1)[i+2])*alpha +                            \
-                             load_macro((src2)[i+2])*beta + gamma);                     \
-            t1 = cast_macro1(load_macro((src1)[i+3])*alpha +                            \
-                             load_macro((src2)[i+3])*beta + gamma);                     \
-                                                                                        \
-            (dst)[i+2] = cast_macro2( t0 );                                             \
-            (dst)[i+3] = cast_macro2( t1 );                                             \
-        }                                                                               \
-                                                                                        \
-        for( ; i < size.width; i++ )                                                    \
-        {                                                                               \
-            worktype t0 = cast_macro1(load_macro((src1)[i])*alpha +                     \
-                                      load_macro((src2)[i])*beta + gamma);              \
-            (dst)[i] = cast_macro2( t0 );                                               \
-        }                                                                               \
-    }                                                                                   \
-                                                                                        \
-    return CV_OK;                                                                       \
+#define ICV_DEF_ADD_WEIGHTED_OP(flavor, arrtype, worktype, load_macro,          \
+                                     cast_macro1, cast_macro2)                  \
+static CvStatus CV_STDCALL                                                      \
+icvAddWeighted_##flavor##_C1R( const arrtype* src1, int step1, double alpha,    \
+                               const arrtype* src2, int step2, double beta,     \
+                               double gamma, arrtype* dst, int step, CvSize size )\
+{                                                                               \
+    step1 /= sizeof(src1[0]); step2 /= sizeof(src2[0]); step /= sizeof(dst[0]); \
+                                                                                \
+    for( ; size.height--; src1 += step1, src2 += step2, dst += step )           \
+    {                                                                           \
+        int i;                                                                  \
+                                                                                \
+        for( i = 0; i <= size.width - 4; i += 4 )                               \
+        {                                                                       \
+            worktype t0 = cast_macro1(load_macro((src1)[i])*alpha +             \
+                                      load_macro((src2)[i])*beta + gamma);      \
+            worktype t1 = cast_macro1(load_macro((src1)[i+1])*alpha +           \
+                                      load_macro((src2)[i+1])*beta + gamma);    \
+                                                                                \
+            (dst)[i] = cast_macro2( t0 );                                       \
+            (dst)[i+1] = cast_macro2( t1 );                                     \
+                                                                                \
+            t0 = cast_macro1(load_macro((src1)[i+2])*alpha +                    \
+                             load_macro((src2)[i+2])*beta + gamma);             \
+            t1 = cast_macro1(load_macro((src1)[i+3])*alpha +                    \
+                             load_macro((src2)[i+3])*beta + gamma);             \
+                                                                                \
+            (dst)[i+2] = cast_macro2( t0 );                                     \
+            (dst)[i+3] = cast_macro2( t1 );                                     \
+        }                                                                       \
+                                                                                \
+        for( ; i < size.width; i++ )                                            \
+        {                                                                       \
+            worktype t0 = cast_macro1(load_macro((src1)[i])*alpha +             \
+                                      load_macro((src2)[i])*beta + gamma);      \
+            (dst)[i] = cast_macro2( t0 );                                       \
+        }                                                                       \
+    }                                                                           \
+                                                                                \
+    return CV_OK;                                                               \
 }
 
 
