@@ -355,7 +355,12 @@ CV_IMPL void cvError( int code, const char* func_name,
             int terminate = context->error_callback( code, func_name, err_msg,
                                                     file_name, line, context->userdata );
             if( terminate )
+            {
+#if !defined WIN32 && !defined WIN64
+                assert(0); // for post-mortem analysis with GDB
+#endif
                 exit(-abs(terminate));
+            }
         }
     }
 }
