@@ -84,7 +84,7 @@ public:
     ~GrFmtBmpWriter();
     
     bool  WriteImage( const uchar* data, int step,
-                      int width, int height, int channels );
+                      int width, int height, int depth, int channels );
 protected:
 
     WLByteStream  m_strm;
