@@ -191,6 +191,139 @@ bool  GrFmtJpegReader::ReadHeader()
     return result;
 }
 
+/***************************************************************************
+ * following code is for supporting MJPEG image files
+ * based on a message of Laurent Pinchart on the video4linux mailing list
+ ***************************************************************************/
+
+/* JPEG DHT Segment for YCrCb omitted from MJPEG data */
+static
+unsigned char my_jpeg_odml_dht[0x1a4] = {
+    0xff, 0xc4, 0x01, 0xa2,
+
+    0x00, 0x00, 0x01, 0x05, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b,
+
+    0x01, 0x00, 0x03, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01,
+    0x00, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b,
+
+    0x10, 0x00, 0x02, 0x01, 0x03, 0x03, 0x02, 0x04, 0x03, 0x05, 0x05, 0x04,
+    0x04, 0x00, 0x00, 0x01, 0x7d,
+    0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12, 0x21, 0x31, 0x41, 0x06,
+    0x13, 0x51, 0x61, 0x07,
+    0x22, 0x71, 0x14, 0x32, 0x81, 0x91, 0xa1, 0x08, 0x23, 0x42, 0xb1, 0xc1,
+    0x15, 0x52, 0xd1, 0xf0,
+    0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0a, 0x16, 0x17, 0x18, 0x19, 0x1a,
+    0x25, 0x26, 0x27, 0x28,
+    0x29, 0x2a, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x43, 0x44, 0x45,
+    0x46, 0x47, 0x48, 0x49,
+    0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x63, 0x64, 0x65,
+    0x66, 0x67, 0x68, 0x69,
+    0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x83, 0x84, 0x85,
+    0x86, 0x87, 0x88, 0x89,
+    0x8a, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a, 0xa2, 0xa3,
+    0xa4, 0xa5, 0xa6, 0xa7,
+    0xa8, 0xa9, 0xaa, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9, 0xba,
+    0xc2, 0xc3, 0xc4, 0xc5,
+    0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8,
+    0xd9, 0xda, 0xe1, 0xe2,
+    0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xf1, 0xf2, 0xf3, 0xf4,
+    0xf5, 0xf6, 0xf7, 0xf8,
+    0xf9, 0xfa,
+
+    0x11, 0x00, 0x02, 0x01, 0x02, 0x04, 0x04, 0x03, 0x04, 0x07, 0x05, 0x04,
+    0x04, 0x00, 0x01, 0x02, 0x77,
+    0x00, 0x01, 0x02, 0x03, 0x11, 0x04, 0x05, 0x21, 0x31, 0x06, 0x12, 0x41,
+    0x51, 0x07, 0x61, 0x71,
+    0x13, 0x22, 0x32, 0x81, 0x08, 0x14, 0x42, 0x91, 0xa1, 0xb1, 0xc1, 0x09,
+    0x23, 0x33, 0x52, 0xf0,
+    0x15, 0x62, 0x72, 0xd1, 0x0a, 0x16, 0x24, 0x34, 0xe1, 0x25, 0xf1, 0x17,
+    0x18, 0x19, 0x1a, 0x26,
+    0x27, 0x28, 0x29, 0x2a, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x43, 0x44,
+    0x45, 0x46, 0x47, 0x48,
+    0x49, 0x4a, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x63, 0x64,
+    0x65, 0x66, 0x67, 0x68,
+    0x69, 0x6a, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7a, 0x82, 0x83,
+    0x84, 0x85, 0x86, 0x87,
+    0x88, 0x89, 0x8a, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9a,
+    0xa2, 0xa3, 0xa4, 0xa5,
+    0xa6, 0xa7, 0xa8, 0xa9, 0xaa, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8,
+    0xb9, 0xba, 0xc2, 0xc3,
+    0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9, 0xca, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6,
+    0xd7, 0xd8, 0xd9, 0xda,
+    0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea, 0xf2, 0xf3, 0xf4,
+    0xf5, 0xf6, 0xf7, 0xf8,
+    0xf9, 0xfa
+};
+
+/*
+ * Parse the DHT table.
+ * This code comes from jpeg6b (jdmarker.c).
+ */
+static
+int my_jpeg_load_dht (struct jpeg_decompress_struct *info, unsigned char *dht,
+              JHUFF_TBL *ac_tables[], JHUFF_TBL *dc_tables[])
+{
+    unsigned int length = (dht[2] << 8) + dht[3] - 2;
+    unsigned int pos = 4;
+    unsigned int count, i;
+    int index;
+
+    JHUFF_TBL **hufftbl;
+    unsigned char bits[17];
+    unsigned char huffval[256];
+
+    while (length > 16)
+    {
+       bits[0] = 0;
+       index = dht[pos++];
+       count = 0;
+       for (i = 1; i <= 16; ++i)
+       {
+           bits[i] = dht[pos++];
+           count += bits[i];
+       }
+       length -= 17;
+
+       if (count > 256 || count > length)
+           return -1;
+    
+       for (i = 0; i < count; ++i)
+           huffval[i] = dht[pos++];
+       length -= count;
+
+       if (index & 0x10)
+       {
+           index -= 0x10;
+           hufftbl = &ac_tables[index];
+       }
+       else
+           hufftbl = &dc_tables[index];
+    
+       if (index < 0 || index >= NUM_HUFF_TBLS)
+           return -1;
+    
+       if (*hufftbl == NULL)
+           *hufftbl = jpeg_alloc_huff_table ((j_common_ptr)info);
+       if (*hufftbl == NULL)
+           return -1;
+
+       memcpy ((*hufftbl)->bits, bits, sizeof (*hufftbl)->bits);
+       memcpy ((*hufftbl)->huffval, huffval, sizeof (*hufftbl)->huffval);
+    }
+    
+    if (length != 0)
+       return -1;
+    
+    return 0;
+}
+
+/***************************************************************************
+ * end of code for supportting MJPEG image files
+ * based on a message of Laurent Pinchart on the video4linux mailing list
+ ***************************************************************************/
 
 bool  GrFmtJpegReader::ReadData( uchar* data, int step, int color )
 {
@@ -206,36 +339,75 @@ bool  GrFmtJpegReader::ReadData( uchar* data, int step, int color )
         
         if( setjmp( jerr->setjmp_buffer ) == 0 )
         {
-            int planes = m_iscolor ? 3 : 1;
-            int width_n = m_width * planes;
+            /* check if this is a mjpeg image format */
+            if ( cinfo->ac_huff_tbl_ptrs[0] == NULL &&
+                cinfo->ac_huff_tbl_ptrs[1] == NULL &&
+                cinfo->dc_huff_tbl_ptrs[0] == NULL &&
+                cinfo->dc_huff_tbl_ptrs[1] == NULL )
+            {
+                /* yes, this is a mjpeg image format, so load the correct
+                huffman table */
+                my_jpeg_load_dht( cinfo,
+                    my_jpeg_odml_dht,
+                    cinfo->ac_huff_tbl_ptrs,
+                    cinfo->dc_huff_tbl_ptrs );
+            }
+
+            if( color > 0 || (m_iscolor && color < 0) )
+            {
+                color = 1;
+                if( cinfo->num_components != 4 )
+                {
+                    cinfo->out_color_space = JCS_RGB;
+                    cinfo->out_color_components = 3;
+                }
+                else
+                {
+                    cinfo->out_color_space = JCS_CMYK;
+                    cinfo->out_color_components = 4;
+                }
+            }
+            else
+            {
+                color = 0;
+                if( cinfo->num_components != 4 )
+                {
+                    cinfo->out_color_space = JCS_GRAYSCALE;
+                    cinfo->out_color_components = 1;
+                }
+                else
+                {
+                    cinfo->out_color_space = JCS_CMYK;
+                    cinfo->out_color_components = 4;
+                }
+            }
 
             jpeg_start_decompress( cinfo );
 
             buffer = (*cinfo->mem->alloc_sarray)((j_common_ptr)cinfo,
-                                              JPOOL_IMAGE, width_n, 1 );
+                                              JPOOL_IMAGE, m_width*4, 1 );
 
             for( ; m_height--; data += step )
             {
                 jpeg_read_scanlines( cinfo, buffer, 1 );
                 if( color )
                 {
-                    if( m_iscolor )
+                    if( cinfo->out_color_components == 3 )
                         icvCvt_RGB2BGR_8u_C3R( buffer[0], 0, data, 0, cvSize(m_width,1) );
                     else
-                        icvCvt_Gray2BGR_8u_C1C3R( buffer[0], 0, data, 0, cvSize(m_width,1) );
+                        icvCvt_CMYK2BGR_8u_C4C3R( buffer[0], 0, data, 0, cvSize(m_width,1) );
                 }
                 else
                 {
-                    if( m_iscolor )
-                        icvCvt_BGR2Gray_8u_C3C1R( buffer[0], 0, data, 0, cvSize(m_width,1), 2 );
-                    else
+                    if( cinfo->out_color_components == 1 )
                         memcpy( data, buffer[0], m_width );
+                    else
+                        icvCvt_CMYK2Gray_8u_C4C1R( buffer[0], 0, data, 0, cvSize(m_width,1) );
                 }
             }
             result = true;
+            jpeg_finish_decompress( cinfo );
         }
-
-        jpeg_finish_decompress( cinfo );
     }
 
     Close();
@@ -256,9 +428,9 @@ GrFmtJpegWriter::~GrFmtJpegWriter()
 
 
 bool  GrFmtJpegWriter::WriteImage( const uchar* data, int step,
-                                   int width, int height, int _channels )
+                                   int width, int height, int /*depth*/, int _channels )
 {
-    const int default_quality = 90;
+    const int default_quality = 95;
     struct jpeg_compress_struct cinfo;
     GrFmtJpegErrorMgr jerr;
 
@@ -461,7 +633,7 @@ static const int idct_prescale[] =
 #define C1_847     fix( 1.847759065f, fixb )
 #define C2_613     fix( 2.613125930f, fixb )
 
-#define fixc       10
+#define fixc       12
 #define b_cb       fix( 1.772, fixc )
 #define g_cb      -fix( 0.34414, fixc )
 #define g_cr      -fix( 0.71414, fixc )
@@ -969,7 +1141,7 @@ void  GrFmtJpegReader::ProcessScan( int* idx, int ns, uchar* data, int step, int
 
     for( mcu = 0;; mcu++ )
     {
-        int  x2, y2, x, y;
+        int  x2, y2, x, y, xc;
         int* cmp;
         uchar* data1;
         
@@ -1034,7 +1206,45 @@ void  GrFmtJpegReader::ProcessScan( int* idx, int ns, uchar* data, int step, int
                     int  shift = h[1]*(y >> y_shift);
                     int* cmpCb = blocks[pos[1]] + shift; 
                     int* cmpCr = blocks[pos[2]] + shift;
-                    for( x = 0; x < x2; x++ )
+                    x = 0;
+                    if( x_shift == 0 )
+                    {
+                        for( ; x < x2; x++ )
+                        {
+                            int Y  = (cmp[x] + 128*4) << fixc;
+                            int Cb = cmpCb[x];
+                            int Cr = cmpCr[x];
+                            int t = (Y + Cb*b_cb) >> (fixc + 2);
+                            data1[x*3] = saturate(t);
+                            t = (Y + Cb*g_cb + Cr*g_cr) >> (fixc + 2);
+                            data1[x*3 + 1] = saturate(t);
+                            t = (Y + Cr*r_cr) >> (fixc + 2);
+                            data1[x*3 + 2] = saturate(t);
+                        }
+                    }
+                    else if( x_shift == 1 )
+                    {
+                        for( xc = 0; x <= x2 - 2; x += 2, xc++ )
+                        {
+                            int Y  = (cmp[x] + 128*4) << fixc;
+                            int Cb = cmpCb[xc];
+                            int Cr = cmpCr[xc];
+                            int t = (Y + Cb*b_cb) >> (fixc + 2);
+                            data1[x*3] = saturate(t);
+                            t = (Y + Cb*g_cb + Cr*g_cr) >> (fixc + 2);
+                            data1[x*3 + 1] = saturate(t);
+                            t = (Y + Cr*r_cr) >> (fixc + 2);
+                            data1[x*3 + 2] = saturate(t);
+                            Y = (cmp[x+1] + 128*4) << fixc;
+                            t = (Y + Cb*b_cb) >> (fixc + 2);
+                            data1[x*3 + 3] = saturate(t);
+                            t = (Y + Cb*g_cb + Cr*g_cr) >> (fixc + 2);
+                            data1[x*3 + 4] = saturate(t);
+                            t = (Y + Cr*r_cr) >> (fixc + 2);
+                            data1[x*3 + 5] = saturate(t);
+                        }
+                    }
+                    for( ; x < x2; x++ )
                     {
                         int Y  = (cmp[x] + 128*4) << fixc;
                         int Cb = cmpCb[x >> x_shift];
@@ -1407,7 +1617,7 @@ static void aan_fdct8x8( int *src, int *dst,
 
 
 bool  GrFmtJpegWriter::WriteImage( const uchar* data, int step,
-                                   int width, int height, int _channels )
+                                   int width, int height, int /*depth*/, int _channels )
 {
     assert( data && width > 0 && height > 0 );
     
@@ -1421,7 +1631,7 @@ bool  GrFmtJpegWriter::WriteImage( const uchar* data, int step,
     //     encode block.
     int x, y;
     int i, j;
-    const int max_quality = 10;
+    const int max_quality = 12;
     int   quality = max_quality;
     WMByteStream& lowstrm = m_strm.m_low_strm;
     int   fdct_qtab[2][64];
