@@ -152,7 +152,7 @@ public:
     ~GrFmtTiffWriter();
 
     bool  WriteImage( const uchar* data, int step,
-                      int width, int height, int channels );
+                      int width, int height, int depth, int channels );
 protected:
 
     WLByteStream  m_strm;
