@@ -204,7 +204,7 @@ CV_IMPL int cvInitSystem( int, char** )
         wndc.lpszClassName = "HighGUI class";
         wndc.lpszMenuName = "HighGUI class";
         wndc.hIcon = LoadIcon(0, IDI_APPLICATION);
-        wndc.hCursor = (HCURSOR)LoadCursor(0, MAKEINTRESOURCE(IDC_CROSS));
+        wndc.hCursor = (HCURSOR)LoadCursor(0, (LPSTR)(size_t)IDC_CROSS );
         wndc.hbrBackground = (HBRUSH)GetStockObject(GRAY_BRUSH);
 
         RegisterClass(&wndc);
@@ -223,6 +223,9 @@ CV_IMPL int cvInitSystem( int, char** )
     return 0;
 }
 
+CV_IMPL int cvStartWindowThread(){
+    return 0;
+}
 
 static CvWindow* icvFindWindowByName( const char* name )
 {
@@ -256,32 +259,40 @@ static const char* icvWindowPosRootKey = "Software\\OpenCV\\HighGUI\\Windows\\";
 // Window positions saving/loading added by Philip Gruebele.
 //<a href="mailto:pgruebele@cox.net">pgruebele@cox.net</a>
 // Restores the window position from the registry saved position.
-static CvPoint
-icvLoadWindowPos( const char* name )
+static void
+icvLoadWindowPos( const char* name, CvRect& rect )
 {
-    CvPoint pos = { CW_USEDEFAULT, CW_USEDEFAULT };
     HKEY hkey;
     char szKey[1024];
     strcpy( szKey, icvWindowPosRootKey );
     strcat( szKey, name );
+
+    rect.x = rect.y = CW_USEDEFAULT;
+    rect.width = rect.height = 320;
+
     if( RegOpenKeyEx(HKEY_CURRENT_USER,szKey,0,KEY_QUERY_VALUE,&hkey) == ERROR_SUCCESS )
     {
         // Yes we are installed.
         DWORD dwType = 0;
         DWORD dwSize = sizeof(int);
 
-        RegQueryValueEx(hkey, "Left", NULL, &dwType, (BYTE*)&pos.x, &dwSize);
-        RegQueryValueEx(hkey, "Top", NULL, &dwType, (BYTE*)&pos.y, &dwSize);
+        RegQueryValueEx(hkey, "Left", NULL, &dwType, (BYTE*)&rect.x, &dwSize);
+        RegQueryValueEx(hkey, "Top", NULL, &dwType, (BYTE*)&rect.y, &dwSize);
+        RegQueryValueEx(hkey, "Width", NULL, &dwType, (BYTE*)&rect.width, &dwSize);
+        RegQueryValueEx(hkey, "Height", NULL, &dwType, (BYTE*)&rect.height, &dwSize);
 
-        if( pos.x != (int)CW_USEDEFAULT && (pos.x < -200 || pos.x > 3000) )
-            pos.x = 100;
-        if( pos.y != (int)CW_USEDEFAULT && (pos.y < -200 || pos.y > 3000) )
-            pos.y = 100;
+        if( rect.x != (int)CW_USEDEFAULT && (rect.x < -200 || rect.x > 3000) )
+            rect.x = 100;
+        if( rect.y != (int)CW_USEDEFAULT && (rect.y < -200 || rect.y > 3000) )
+            rect.y = 100;
+
+        if( rect.width != (int)CW_USEDEFAULT && (rect.width < 0 || rect.width > 3000) )
+            rect.width = 100;
+        if( rect.height != (int)CW_USEDEFAULT && (rect.height < 0 || rect.height > 3000) )
+            rect.height = 100;
 
         RegCloseKey(hkey);
     }
-
-    return pos;
 }
 
 
@@ -289,7 +300,7 @@ icvLoadWindowPos( const char* name )
 //<a href="mailto:pgruebele@cox.net">pgruebele@cox.net</a>
 // philipg.  Saves the window position in the registry
 static void
-icvSaveWindowPos( const char* name, CvPoint pos )
+icvSaveWindowPos( const char* name, CvRect rect )
 {
     static const DWORD MAX_RECORD_COUNT = 100;
     HKEY hkey;
@@ -343,8 +354,10 @@ icvSaveWindowPos( const char* name, CvPoint pos )
             return;
     }
     
-    RegSetValueEx(hkey, "Left", 0, REG_DWORD, (BYTE*)&pos.x, sizeof(pos.x));
-    RegSetValueEx(hkey, "Top", 0, REG_DWORD, (BYTE*)&pos.y, sizeof(pos.y));
+    RegSetValueEx(hkey, "Left", 0, REG_DWORD, (BYTE*)&rect.x, sizeof(rect.x));
+    RegSetValueEx(hkey, "Top", 0, REG_DWORD, (BYTE*)&rect.y, sizeof(rect.y));
+    RegSetValueEx(hkey, "Width", 0, REG_DWORD, (BYTE*)&rect.width, sizeof(rect.width));
+    RegSetValueEx(hkey, "Height", 0, REG_DWORD, (BYTE*)&rect.height, sizeof(rect.height));
     RegCloseKey(hkey);
 }
 
@@ -360,7 +373,7 @@ CV_IMPL int cvNamedWindow( const char* name, int flags )
     CvWindow* window;
     DWORD defStyle = WS_VISIBLE | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_SYSMENU;
     int len;
-    CvPoint pos;
+    CvRect rect;
 
     cvInitSystem(0,0);
 
@@ -377,17 +390,17 @@ CV_IMPL int cvNamedWindow( const char* name, int flags )
     if( (flags & CV_WINDOW_AUTOSIZE) == 0 )
         defStyle |= WS_SIZEBOX;
 
-    pos = icvLoadWindowPos( name );
+    icvLoadWindowPos( name, rect );
 
     mainhWnd = CreateWindow( "Main HighGUI class", name, defStyle | WS_OVERLAPPED,
-                             pos.x, pos.y, 320, 320, 0, 0, hg_hinstance, 0 );
+                             rect.x, rect.y, rect.width, rect.height, 0, 0, hg_hinstance, 0 );
     if( !mainhWnd )
         CV_ERROR( CV_StsError, "Frame window can not be created" );
 
     ShowWindow(mainhWnd, SW_SHOW);
 
     hWnd = CreateWindow("HighGUI class", "", defStyle | WS_CHILD | WS_SIZEBOX,
-                        CW_USEDEFAULT, 0, 320, 320, mainhWnd, 0, hg_hinstance, 0);
+                        CW_USEDEFAULT, 0, rect.width, rect.height, mainhWnd, 0, hg_hinstance, 0);
     if( !hWnd )
         CV_ERROR( CV_StsError, "Frame window can not be created" );
 
@@ -435,7 +448,8 @@ static void icvRemoveWindow( CvWindow* window )
     RECT wrect;
 
     GetWindowRect( window->frame, &wrect );
-    icvSaveWindowPos( window->name, cvPoint(wrect.left, wrect.top) );
+    icvSaveWindowPos( window->name, cvRect(wrect.left, wrect.top,
+        wrect.right-wrect.left, wrect.bottom-wrect.top) );
 
     icvSetWindowLongPtr( window->hwnd, CV_USERDATA, 0 );
     icvSetWindowLongPtr( window->frame, CV_USERDATA, 0 );
@@ -460,11 +474,11 @@ static void icvRemoveWindow( CvWindow* window )
     {
         CvTrackbar* next = trackbar->next;
         icvSetWindowLongPtr( trackbar->hwnd, CV_USERDATA, 0 );
-        cvFree( (void**)&trackbar );
+        cvFree( &trackbar );
         trackbar = next;
     }
 
-    cvFree( (void**)&window );
+    cvFree( &window );
 }
 
 
@@ -536,6 +550,11 @@ static bool icvGetBitmapData( CvWindow* window, SIZE* size, int* channels, void*
     BITMAP bmp;
     GdiFlush();
     HGDIOBJ h = GetCurrentObject( window->dc, OBJ_BITMAP );
+    if( size )
+        size->cx = size->cy = 0;
+    if( data )
+        *data = 0;
+
     if (h == NULL)
         return true;
     if (GetObject(h, sizeof(bmp), &bmp) == 0)
@@ -565,7 +584,7 @@ static void icvUpdateWindowPos( CvWindow* window )
     if( (window->flags & CV_WINDOW_AUTOSIZE) && window->image )
     {
         int i;
-        SIZE size;
+        SIZE size = {0,0};
         icvGetBitmapData( window, &size, 0, 0 );
 
         // Repeat two times because after the first resizing of the mainhWnd window
@@ -605,7 +624,7 @@ cvShowImage( const char* name, const CvArr* arr )
     const int channels0 = 3;
     int origin = 0;
     CvMat stub, dst, *image;
-    bool changed_size = false;
+    bool changed_size = false; // philipg
 
     if( !name )
         CV_ERROR( CV_StsNullPtr, "NULL name" );
@@ -619,8 +638,10 @@ cvShowImage( const char* name, const CvArr* arr )
 
     CV_CALL( image = cvGetMat( arr, &stub ));
 
-    if( window->image )
-        icvGetBitmapData( window, &size, &channels, &dst_ptr );
+    if (window->image)
+        // if there is something wrong with these system calls, we cannot display image...
+        if (icvGetBitmapData( window, &size, &channels, &dst_ptr ))
+            return;
 
     if( size.cx != image->width || size.cy != image->height || channels != channels0 )
     {
@@ -805,6 +826,10 @@ MainWindowProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
 
             if(ret != NULLREGION && ret != ERROR)
                 FillRgn(hdc, rgn, (HBRUSH)icvGetClassLongPtr(hwnd, CV_HBRBACKGROUND));
+
+            DeleteObject(rgn);
+            DeleteObject(rgn1);
+            DeleteObject(rgn2);
         }
         return 1;
     }
@@ -849,7 +874,7 @@ static LRESULT CALLBACK HighGUIProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM
         {
             POINT pt;
             RECT rect;
-            SIZE size;
+            SIZE size = {0,0};
 
             int flags = (wParam & MK_LBUTTON ? CV_EVENT_FLAG_LBUTTON : 0)|
                         (wParam & MK_RBUTTON ? CV_EVENT_FLAG_RBUTTON : 0)|
@@ -888,7 +913,7 @@ static LRESULT CALLBACK HighGUIProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM
         if(window->image != 0)
         {
             int nchannels = 3;
-            SIZE size;
+            SIZE size = {0,0};
             PAINTSTRUCT paint;
             HDC hdc;
             RGBQUAD table[256];
@@ -1256,14 +1281,16 @@ cvCreateTrackbar( const char* trackbar_name, const char* window_name,
         tbs.iBitmap = 0;
         tbs.idCommand = bcount; // Set button id to it's number
         tbs.fsState = TBSTATE_ENABLED;
-#if !defined WIN64 && !defined EM64T
+#if 0/*!defined WIN64 && !defined EM64T*/
         tbs.fsStyle = 0;
         tbs.iString = 0;
 #else
 #ifndef TBSTYLE_AUTOSIZE
 #define TBSTYLE_AUTOSIZE        0x0010
+#define TBSTYLE_GROUP           0x0004
 #endif
-        tbs.fsStyle = TBSTYLE_AUTOSIZE;
+        //tbs.fsStyle = TBSTYLE_AUTOSIZE;
+        tbs.fsStyle = TBSTYLE_GROUP;
         tbs.iString = (INT_PTR)trackbar_text;
 #endif
         SendMessage(window->toolbar.toolbar, TB_ADDBUTTONS, 1, (LPARAM)&tbs);
