@@ -179,7 +179,7 @@ int main( int argc, char** argv )
             
             if( backproject_mode )
                 cvCvtColor( backproject, image, CV_GRAY2BGR );
-            if( image->origin )
+            if( !image->origin )
                 track_box.angle = -track_box.angle;
             cvEllipseBox( image, track_box, CV_RGB(255,0,0), 3, CV_AA, 0 );
         }
@@ -195,9 +195,9 @@ int main( int argc, char** argv )
         cvShowImage( "Histogram", histimg );
 
         c = cvWaitKey(10);
-        if( c == 27 )
+        if( (char) c == 27 )
             break;
-        switch( c )
+        switch( (char) c )
         {
         case 'b':
             backproject_mode ^= 1;
