@@ -5,6 +5,7 @@
 #ifndef _EiC
 #include "cv.h"
 #include "highgui.h"
+#include <math.h>
 #endif
 
 #define w 500
@@ -26,11 +27,12 @@ void on_trackbar(int pos)
 
 int main( int argc, char** argv )
 {
-    int i;
+    int i, j;
     CvMemStorage* storage = cvCreateMemStorage(0);
     IplImage* img = cvCreateImage( cvSize(w,w), 8, 1 );
 
     cvZero( img );
+
     for( i=0; i < 6; i++ )
     {
         int dx = (i%2)*250 - 30;
@@ -38,6 +40,18 @@ int main( int argc, char** argv )
         CvScalar white = cvRealScalar(255);
         CvScalar black = cvRealScalar(0);
 
+        if( i == 0 )
+        {
+            for( j = 0; j <= 10; j++ )
+            {
+                double angle = (j+5)*CV_PI/21;
+                cvLine(img, cvPoint(cvRound(dx+100+j*10-80*cos(angle)),
+                    cvRound(dy+100-90*sin(angle))),
+                    cvPoint(cvRound(dx+100+j*10-30*cos(angle)),
+                    cvRound(dy+100-30*sin(angle))), white, 1, 8, 0);
+            }
+        }
+
         cvEllipse( img, cvPoint(dx+150, dy+100), cvSize(100,70), 0, 0, 360, white, -1, 8, 0 );
         cvEllipse( img, cvPoint(dx+115, dy+70), cvSize(30,20), 0, 0, 360, black, -1, 8, 0 );
         cvEllipse( img, cvPoint(dx+185, dy+70), cvSize(30,20), 0, 0, 360, black, -1, 8, 0 );
@@ -65,6 +79,8 @@ int main( int argc, char** argv )
     
     on_trackbar(0);
     cvWaitKey(0);
+    cvReleaseMemStorage( &storage );
+    cvReleaseImage( &img );
 
     return 0;
 }
