@@ -116,7 +116,7 @@ void CV_ImgWarpBaseTestImpl::get_test_array_types_and_sizes( int test_case_idx,
                                                 CvSize** sizes, int** types )
 {
     CvRNG* rng = ts->get_rng();
-    int depth = test_case_idx*3/test_case_count;
+    int depth = cvTsRandInt(rng) % 3;
     int cn = cvTsRandInt(rng) % 3 + 1;
     CvArrTest::get_test_array_types_and_sizes( test_case_idx, sizes, types );
     depth = depth == 0 ? CV_8U : depth == 1 ? CV_16U : CV_32F;
@@ -153,7 +153,7 @@ int CV_ImgWarpBaseTestImpl::prepare_test_case( int test_case_idx )
         test_mat[INPUT_OUTPUT][0].rows >= img->rows )
         space_scale = spatial_scale_zoom;
 
-    buffer = (float*)malloc( img->cols*cn*sizeof(buffer[0]) );
+    buffer = (float*)cvAlloc( img->cols*cn*sizeof(buffer[0]) );
     
     for( i = 0; i < img->rows; i++ )
     {
@@ -211,7 +211,7 @@ int CV_ImgWarpBaseTestImpl::prepare_test_case( int test_case_idx )
         }
     }
 
-    free( buffer );
+    cvFree( &buffer );
 
     return code;
 }
@@ -1138,7 +1138,7 @@ void CV_RemapTest::prepare_to_validation( int /*test_case_idx*/ )
 {
     CvMat* dst = &test_mat[REF_INPUT_OUTPUT][0];
     CvMat* dst0 = &test_mat[INPUT_OUTPUT][0];
-    int nr = interpolation == CV_INTER_CUBIC ? 3 : 2, nc = nr;
+    int nr = interpolation == CV_INTER_CUBIC ? 3 : 3, nc = nr;
     CvMat part;
     cvTsRemap( &test_mat[INPUT][0], dst, dst0,
                &test_mat[INPUT][1], &test_mat[INPUT][2],
@@ -1349,6 +1349,9 @@ void CV_UndistortTest::prepare_to_validation( int /*test_case_idx*/ )
     cvTsZero( &part );
     cvGetCols( dst0, &part, dst->cols - nc, dst->cols ); 
     cvTsZero( &part );
+
+    cvReleaseMat( &mapx );
+    cvReleaseMat( &mapy );
 }
 
 
@@ -1628,7 +1631,7 @@ void CV_GetRectSubPixTest::get_test_array_types_and_sizes( int test_case_idx, Cv
 {
     CvRNG* rng = ts->get_rng();
     CV_ImgWarpBaseTest::get_test_array_types_and_sizes( test_case_idx, sizes, types );
-    int src_depth = test_case_idx*2/test_case_count, dst_depth;
+    int src_depth = cvTsRandInt(rng) % 2, dst_depth;
     int cn = cvTsRandInt(rng) % 2 ? 3 : 1;
     CvSize src_size, dst_size;
     
@@ -1787,7 +1790,7 @@ void CV_GetQuadSubPixTest::get_test_array_types_and_sizes( int test_case_idx, Cv
     CV_ImgWarpBaseTest::get_test_array_types_and_sizes( test_case_idx, sizes, types );
     CvSize sz = sizes[INPUT][0], dsz;
     CvRNG* rng = ts->get_rng();
-    int msz, src_depth = test_case_idx*2/test_case_count, dst_depth;
+    int msz, src_depth = cvTsRandInt(rng) % 2, dst_depth;
     int cn = cvTsRandInt(rng) % 2 ? 3 : 1;
     
     dst_depth = src_depth = src_depth == 0 ? CV_8U : CV_32F;
